__version__ = "0.0.3"

__title__ = "thumbor-gcs"
__description__ = "Thumbor Loader and Result Storage for Google Cloud Storage(GCS)"
__uri__ = "https://github.com/jjonline/thumbor-gcs"
__doc__ = __description__ + " <" + __uri__ + ">"

__author_name__ = "Jea Yang"
__author_email__ = "jjonline@jjonline.cn"
__maintainer_name__ = "Jea Yang"
__maintainer_email__ = "jjonline@jjonline.cn"

__license__ = "Apache-2.0 license"
__copyright__ = "Copyright (c) 2022 Jea Yang"
